# noqa: D100

extensions = [
    "atsphinx.typst",
]

typst_documents = [
    {
        "entry": "index",
        "filename": "index",
        "theme": "manual",
        "title": "Test documentation",
    }
]
